// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.
import { __asyncValues, __awaiter } from "tslib";
// This sample shows you how to set a configuration setting to read-only. 
// This can help prevent accidental deletion or modification of a setting.
// NOTE: replace with import { AppConfigurationClient } from "@azure/app-configuration"
// in a standalone project
import { AppConfigurationClient } from "../src";
export function run() {
    return __awaiter(this, void 0, void 0, function* () {
        console.log("Running setReadOnly sample");
        // You will need to set this environment variable
        const connectionString = process.env["AZ_CONFIG_CONNECTION"];
        const client = new AppConfigurationClient(connectionString);
        const readOnlySampleKey = "readOnlySampleKey";
        yield cleanupSampleValues([readOnlySampleKey], client);
        console.log("Creating a new key. By default keys are writeable");
        yield client.addConfigurationSetting({ key: readOnlySampleKey, label: "a label", value: "Initial value" });
        // now we'd like to prevent future modifications - let's set the key/label to read-only
        console.log("Setting a key to read-only. Any modifications will fail");
        yield client.setReadOnly({ key: readOnlySampleKey, label: "a label" }, true);
        // any modifications to the key will now throw errors
        try {
            console.log("Attempting to modify a read-only setting");
            yield client.setConfigurationSetting({ key: readOnlySampleKey, label: "a label", value: "new value" });
        }
        catch (err) {
            console.log(`Error gets thrown - can't modify a read-only setting`);
        }
        // clients that read from the key are unaffected
        yield client.getConfigurationSetting({ key: readOnlySampleKey, label: "a label" });
        // to make a key writable again we can clear the read-only status
        console.log("Clearing the read-only status on the key so we can update the value");
        yield client.setReadOnly({ key: readOnlySampleKey, label: "a label" }, false);
        // and now clients can change the value again
        const updatedSetting = yield client.setConfigurationSetting({ key: readOnlySampleKey, label: "a label", value: "new value" });
        console.log(`Updated value is ${updatedSetting.value}`);
        yield cleanupSampleValues([readOnlySampleKey], client);
    });
}
function cleanupSampleValues(keys, client) {
    var e_1, _a;
    return __awaiter(this, void 0, void 0, function* () {
        const existingSettings = yield client.listConfigurationSettings({
            keyFilter: keys.join(',')
        });
        try {
            for (var existingSettings_1 = __asyncValues(existingSettings), existingSettings_1_1; existingSettings_1_1 = yield existingSettings_1.next(), !existingSettings_1_1.done;) {
                const setting = existingSettings_1_1.value;
                yield client.setReadOnly(setting, false);
                yield client.deleteConfigurationSetting({ key: setting.key, label: setting.label });
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (existingSettings_1_1 && !existingSettings_1_1.done && (_a = existingSettings_1.return)) yield _a.call(existingSettings_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    });
}
// If you want to run this sample from a console
// uncomment these lines so run() will get called
// run().catch(err => {
//     console.log(`ERROR: ${err}`);
// });
//# sourceMappingURL=setReadOnlySample.js.map