// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import { createHash, createHmac } from "crypto";
import { URLBuilder } from "@azure/core-http";
/**
 * @internal
 * @ignore
 */
export class AppConfigCredential {
    constructor(credential, secret) {
        this.credential = credential;
        this.secret = secret;
    }
    /**
     * Signs a request with the values provided in the credential and secret parameter.
     *
     * @param {WebResource} webResource The WebResource to be signed.
     * @returns {Promise<WebResource>} The signed request object.
     */
    signRequest(webResource) {
        const verb = webResource.method.toUpperCase();
        const utcNow = new Date().toUTCString();
        const contentHash = createHash("sha256")
            .update(webResource.body || "")
            .digest("base64");
        const signedHeaders = "x-ms-date;host;x-ms-content-sha256";
        const url = URLBuilder.parse(webResource.url);
        const query = url.getQuery();
        const urlPathAndQuery = `${url.getPath()}${query ? "?" + query : ""}`;
        const stringToSign = `${verb}\n${urlPathAndQuery}\n${utcNow};${url.getHost()};${contentHash}`;
        const decodedSecret = Buffer.from(this.secret, "base64");
        var signature = createHmac("sha256", decodedSecret)
            .update(stringToSign)
            .digest("base64");
        webResource.headers.set("x-ms-date", utcNow);
        webResource.headers.set("x-ms-content-sha256", contentHash);
        webResource.headers.set("Authorization", `HMAC-SHA256 Credential=${this.credential}, SignedHeaders=${signedHeaders}, Signature=${signature}`);
        return Promise.resolve(webResource);
    }
}
//# sourceMappingURL=appConfigCredential.js.map