import { __awaiter } from "tslib";
import { createAppConfigurationClientForTests, deleteKeyCompletely, assertThrowsRestError } from "./testHelpers";
import * as assert from "assert";
// There's been discussion on other teams about what errors are thrown when. This
// is the file where I've documented the throws/notThrows cases to make coordination
// with other teams simpler. (there's redundancy with other parts of the test suite but
// that's okay)
describe("Various error cases", () => {
    let client;
    const nonMatchingETag = "never-match-etag";
    before(function () {
        client = createAppConfigurationClientForTests() || this.skip();
    });
    describe("throws", () => {
        let addedSetting;
        let nonExistentKey;
        beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
            addedSetting = yield client.addConfigurationSetting({
                key: `etags-${Date.now()}`,
                value: "world"
            });
            nonExistentKey = "non-existent key " + addedSetting.key;
        }));
        afterEach(() => __awaiter(void 0, void 0, void 0, function* () {
            yield deleteKeyCompletely([addedSetting.key], client);
        }));
        it("get: Non-existent key throws 404", () => __awaiter(void 0, void 0, void 0, function* () {
            yield assertThrowsRestError(() => client.getConfigurationSetting({ key: nonExistentKey }), 404);
        }));
        it("add: Setting already exists throws 412", () => __awaiter(void 0, void 0, void 0, function* () {
            yield assertThrowsRestError(() => client.addConfigurationSetting(addedSetting), 412);
        }));
        it("set: Existing key, (onlyIfUnchanged) throws 412", () => __awaiter(void 0, void 0, void 0, function* () {
            yield assertThrowsRestError(() => client.setConfigurationSetting(Object.assign(Object.assign({}, addedSetting), { etag: nonMatchingETag // purposefully make the etag not match the server
             }), { onlyIfUnchanged: true }), 412);
        }));
        it("set: trying to modify a read-only setting throws 409", () => __awaiter(void 0, void 0, void 0, function* () {
            yield client.setReadOnly(addedSetting, true);
            yield assertThrowsRestError(() => client.setConfigurationSetting(addedSetting), 409);
        }));
        it("delete: key that is set to read-only throws 409", () => __awaiter(void 0, void 0, void 0, function* () {
            yield client.setReadOnly(addedSetting, true);
            yield assertThrowsRestError(() => __awaiter(void 0, void 0, void 0, function* () { return client.deleteConfigurationSetting(addedSetting); }), 409);
            yield client.setReadOnly(addedSetting, false);
        }));
    });
    describe("doesn't throw", () => {
        let addedSetting;
        let nonExistentKey;
        beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
            // same setup for all tests:
            // key: hello{date}, value: world
            // the 'no label' value for 'hello'
            addedSetting = yield client.addConfigurationSetting({
                key: `etags-${Date.now()}`,
                value: "world"
            });
            nonExistentKey = "bogus key " + addedSetting.key;
        }));
        afterEach(() => __awaiter(void 0, void 0, void 0, function* () {
            yield deleteKeyCompletely([addedSetting.key], client);
        }));
        it("get: value is unchanged from etag (304) using ifNoneMatch, sets all properties to undefined", () => __awaiter(void 0, void 0, void 0, function* () {
            const response = yield client.getConfigurationSetting(addedSetting, {
                onlyIfChanged: true
            });
            assert.equal(304, response.statusCode);
            assert.ok(!response.value);
        }));
        it("delete: non-existent key (no etag)", () => __awaiter(void 0, void 0, void 0, function* () {
            yield client.deleteConfigurationSetting({ key: nonExistentKey });
        }));
    });
});
//# sourceMappingURL=throwOrNotThrow.spec.js.map