/// <reference lib="esnext.asynciterable" />
import { HttpResponse } from '@azure/core-http';
import { OperationOptions } from '@azure/core-http';
import { PagedAsyncIterableIterator } from '@azure/core-paging';
import { TokenCredential } from '@azure/identity';
import { UserAgentOptions } from '@azure/core-http';

/**
 * Options used when adding a ConfigurationSetting.
 */
export declare interface AddConfigurationSettingOptions extends OperationOptions {
}

/**
 * Parameters for adding a new configuration setting
 */
export declare interface AddConfigurationSettingParam extends ConfigurationSettingParam {
}

/**
 * Response from adding a ConfigurationSetting.
 */
export declare interface AddConfigurationSettingResponse extends ConfigurationSetting, SyncTokenHeaderField, HttpResponseField<SyncTokenHeaderField> {
}

/**
 * Client for the Azure App Configuration service.
 */
export declare class AppConfigurationClient {
    private client;
    private spanner;
    /**
     * Initializes a new instance of the AppConfigurationClient class.
     * @param connectionString Connection string needed for a client to connect to Azure.
     * @param options Options for the AppConfigurationClient.
     */
    constructor(connectionString: string, options?: AppConfigurationClientOptions);
    /**
     * Initializes a new instance of the AppConfigurationClient class using
     * a TokenCredential.
     * @param endpoint The endpoint of the App Configuration service (ex: https://sample.azconfig.io).
     * @param tokenCredential An object that implements the `TokenCredential` interface used to authenticate requests to the service. Use the @azure/identity package to create a credential that suits your needs.
     * @param options Options for the AppConfigurationClient.
     */
    constructor(endpoint: string, tokenCredential: TokenCredential, options?: AppConfigurationClientOptions);
    /**
     * Add a setting into the Azure App Configuration service, failing if it
     * already exists.
     *
     * Example usage:
     * ```ts
     * const result = await client.addConfigurationSetting({ key: "MyKey", label: "MyLabel", value: "MyValue" });
     * ```
     * @param configurationSetting A configuration setting.
     * @param options Optional parameters for the request.
     */
    addConfigurationSetting(configurationSetting: AddConfigurationSettingParam, options?: AddConfigurationSettingOptions): Promise<AddConfigurationSettingResponse>;
    /**
     * Delete a setting from the Azure App Configuration service
     *
     * Example usage:
     * ```ts
     * const deletedSetting = await client.deleteConfigurationSetting({ key: "MyKey", label: "MyLabel" });
     * ```
     * @param id The id of the configuration setting to delete.
     * @param options Optional parameters for the request (ex: etag, label)
     */
    deleteConfigurationSetting(id: ConfigurationSettingId, options?: DeleteConfigurationSettingOptions): Promise<DeleteConfigurationSettingResponse>;
    /**
     * Gets a setting from the Azure App Configuration service.
     *
     * Example code:
     * ```ts
     * const setting = await client.getConfigurationSetting({ key: "MyKey", label: "MyLabel" });
     * ```
     * @param id The id of the configuration setting to get.
     * @param options Optional parameters for the request.
     */
    getConfigurationSetting(id: ConfigurationSettingId, options?: GetConfigurationSettingOptions): Promise<GetConfigurationSettingResponse>;
    /**
     * Lists settings from the Azure App Configuration service, optionally
     * filtered by key names, labels and accept datetime.
     *
     * Example code:
     * ```ts
     * const allSettingsWithLabel = await client.listConfigurationSettings({ labels: [ "MyLabel" ] });
     * ```
     * @param options Optional parameters for the request.
     */
    listConfigurationSettings(options?: ListConfigurationSettingsOptions): PagedAsyncIterableIterator<ConfigurationSetting, ListConfigurationSettingPage>;
    private getListConfigurationSettingsIterator;
    private listConfigurationSettingsByPage;
    private createListConfigurationPageFromResponse;
    /**
     * Lists revisions of a set of keys, optionally filtered by key names,
     * labels and accept datetime.
     *
     * Example code:
     * ```ts
     * const revisionsIterator = await client.listRevisions({ keys: ["MyKey"] });
     * ```
     * @param options Optional parameters for the request.
     */
    listRevisions(options?: ListRevisionsOptions): PagedAsyncIterableIterator<ConfigurationSetting, ListRevisionsPage>;
    private getListRevisionsIterator;
    private listRevisionsByPage;
    /**
     * Sets the value of a key in the Azure App Configuration service, allowing for an optional etag.
     * @param key The name of the key.
     * @param configurationSetting A configuration value.
     * @param options Optional parameters for the request.
     *
     * Example code:
     * ```ts
     * await client.setConfigurationSetting({ key: "MyKey", value: "MyValue" });
     * ```
     */
    setConfigurationSetting(configurationSetting: SetConfigurationSettingParam, options?: SetConfigurationSettingOptions): Promise<SetConfigurationSettingResponse>;
    /**
     * Sets or clears a key's read-only status.
     * @param id The id of the configuration setting to modify.
     */
    setReadOnly(id: ConfigurationSettingId, readOnly: boolean, options?: SetReadOnlyOptions): Promise<SetReadOnlyResponse>;
}

/**
 * Provides configuration options for AppConfigurationClient.
 */
export declare interface AppConfigurationClientOptions {
    /**
     * Options for adding user agent details to outgoing requests.
     */
    userAgentOptions?: UserAgentOptions;
}

/**
 * Configuration setting with extra metadata from the server, indicating
 * its etag, whether it is currently readOnly and when it was last modified.
 */
export declare interface ConfigurationSetting extends ConfigurationSettingParam {
    /**
     * Whether or not the setting is read-only
     */
    isReadOnly: boolean;
    /**
     * The date when this setting was last modified
     */
    lastModified?: Date;
}

/**
 * Fields that uniquely identify a configuration setting
 */
export declare interface ConfigurationSettingId {
    /**
     * The key for this setting
     */
    key: string;
    /**
     * The label for this setting. Leaving this undefined means this
     * setting does not have a label.
     */
    label?: string;
    /**
     * The etag for this setting
     */
    etag?: string;
}

/**
 * Necessary fields for updating or creating a new configuration setting
 */
export declare interface ConfigurationSettingParam extends ConfigurationSettingId {
    /**
     * The content type of the setting's value
     */
    contentType?: string;
    /**
     * The setting's value
     */
    value?: string;
    /**
     * Tags for this key
     */
    tags?: {
        [propertyName: string]: string;
    };
}

/**
 * Standard base response for getting, deleting or updating a configuration setting
 */
export declare type ConfigurationSettingResponse<HeadersT> = ConfigurationSetting & HttpResponseField<HeadersT> & Pick<HeadersT, Exclude<keyof HeadersT, "eTag">>;

/**
 * Options for deleting a ConfigurationSetting.
 */
export declare interface DeleteConfigurationSettingOptions extends HttpOnlyIfUnchangedField, OperationOptions {
}

/**
 * Response from deleting a ConfigurationSetting.
 */
export declare interface DeleteConfigurationSettingResponse extends SyncTokenHeaderField, HttpResponseFields, HttpResponseField<SyncTokenHeaderField> {
}

/**
 * Headers from getting a ConfigurationSetting.
 */
export declare interface GetConfigurationHeaders extends SyncTokenHeaderField {
}

/**
 * Options for getting a ConfigurationSetting.
 */
export declare interface GetConfigurationSettingOptions extends OperationOptions, HttpOnlyIfChangedField, OptionalFields {
    /**
     * Requests the server to respond with the state of the resource at the specified time.
     */
    acceptDateTime?: Date;
}

/**
 * Response from retrieving a ConfigurationSetting.
 */
export declare interface GetConfigurationSettingResponse extends ConfigurationSetting, GetConfigurationHeaders, HttpResponseFields, HttpResponseField<GetConfigurationHeaders> {
}

/**
 * Options used to provide if-none-match for an HTTP request
 */
export declare interface HttpOnlyIfChangedField {
    /**
     * Used to perform an operation only if the targeted resource's etag does not match the value
     * provided.
     */
    onlyIfChanged?: boolean;
}

/**
 * Options used to provide if-match for an HTTP request
 */
export declare interface HttpOnlyIfUnchangedField {
    /**
     * Used to perform an operation only if the targeted resource's etag matches the value provided.
     */
    onlyIfUnchanged?: boolean;
}

/**
 * HTTP response related information - headers and raw body.
 */
export declare interface HttpResponseField<HeadersT> {
    /**
     * The underlying HTTP response.
     */
    _response: HttpResponse & {
        /**
         * The parsed HTTP response headers.
         */
        parsedHeaders: HeadersT;
        /**
         * The response body as text (string format)
         */
        bodyAsText: string;
    };
}

/**
 * Fields that are hoisted up  from the _response field of the object
 * Used in cases where individual HTTP response fields are important for
 * the user to use in common-use cases like handling http status codes 204 or 304.
 */
export declare interface HttpResponseFields {
    /**
     * The HTTP status code for the response
     */
    statusCode: number;
}

/**
 * A page of configuration settings and the corresponding HTTP response
 */
export declare interface ListConfigurationSettingPage extends HttpResponseField<SyncTokenHeaderField> {
    /**
     * The configuration settings for this page of results.
     */
    items: ConfigurationSetting[];
}

/**
 * Options for listConfigurationSettings that allow for filtering based on keys, labels and other fields.
 * Also provides `fields` which allows you to selectively choose which fields are populated in the
 * result.
 */
export declare interface ListConfigurationSettingsOptions extends OperationOptions, ListSettingsOptions {
}

/**
 * Options for listRevisions that allow for filtering based on keys, labels and other fields.
 * Also provides `fields` which allows you to selectively choose which fields are populated in the
 * result.
 */
export declare interface ListRevisionsOptions extends OperationOptions, ListSettingsOptions {
}

/**
 * A page of configuration settings and the corresponding HTTP response
 */
export declare interface ListRevisionsPage extends HttpResponseField<SyncTokenHeaderField> {
    /**
     * The configuration settings for this page of results.
     */
    items: ConfigurationSetting[];
}

/**
 * Common options for 'list' style APIs in AppConfig used to specify wildcards as well as
 * the accept date time header.
 */
export declare interface ListSettingsOptions extends OptionalFields {
    /**
     * Requests the server to respond with the state of the resource at the specified time.
     */
    acceptDateTime?: Date;
    /**
     * Filters for keys. There are two types of matching:
     *
     * 1. Exact matching. Up to 5 key names are allowed, separated by commas (',')
     * 2. Wildcard matching. A single wildcard expression can be specified.
     *
     *    | Value        | Matches                               |
     *    |--------------|---------------------------------------|
     *    | omitted or * | Matches any key                       |
     *    | abc          | Matches a key named abc               |
     *    | abc*         | Matches key names that start with abc |
     *    | *abc         | Matches key names that end with abc   |
     *    | *abc*        | Matches key names that contain abc    |
     *
     * These characters are reserved and must be prefixed with backslash in order
     * to be specified: * or \ or ,
     */
    keyFilter?: string;
    /**
     * Filters for labels. There are two types of matching:
     *
     * 1. Exact matching. Up to 5 labels are allowed, separated by commas (',')
     * 2. Wildcard matching. A single wildcard expression can be specified.
     *
     *    | Value        | Matches                                           |
     *    |--------------|---------------------------------------------------|
     *    | omitted or * | Matches any key                                   |
     *    | %00          | Matches any key without a label                   |
     *    | prod         | Matches a key with label named prod               |
     *    | prod*        | Matches key with label names that start with prod |
     *    | *prod        | Matches key with label names that end with prod   |
     *    | *prod*       | Matches key with label names that contain prod    |
     *
     * These characters are reserved and must be prefixed with backslash in order
     * to be specified: * or \ or ,
     */
    labelFilter?: string;
}

/**
 * Used when the API supports selectively returning fields.
 */
export declare interface OptionalFields {
    /**
     * Which fields to return for each ConfigurationSetting
     */
    fields?: (keyof ConfigurationSetting)[];
}

/**
 * Options used when saving a ConfigurationSetting.
 */
export declare interface SetConfigurationSettingOptions extends HttpOnlyIfUnchangedField, OperationOptions {
}

/**
 * Parameters for creating or updating a new configuration setting
 */
export declare interface SetConfigurationSettingParam extends ConfigurationSettingParam {
}

/**
 * Response from setting a ConfigurationSetting.
 */
export declare interface SetConfigurationSettingResponse extends ConfigurationSetting, SyncTokenHeaderField, HttpResponseField<SyncTokenHeaderField> {
}

/**
 * Options for setReadOnly
 */
export declare interface SetReadOnlyOptions extends HttpOnlyIfUnchangedField, OperationOptions {
}

/**
 * Response when setting a value to read-only.
 */
export declare interface SetReadOnlyResponse extends ConfigurationSetting, SyncTokenHeaderField, HttpResponseField<SyncTokenHeaderField> {
}

/**
 * Sync token header field
 */
export declare interface SyncTokenHeaderField {
    /**
     * Enables real-time consistency between requests by providing the returned value in the next
     * request made to the server.
     */
    syncToken?: string;
}

export { }
